@@ -45,6 +45,8 @@
 
 using namespace std;
 
+// From bpp-core:
+#include <Bpp/Version.h>
 #include <Bpp/Numeric/Prob/DiscreteDistribution.h>
 #include <Bpp/Numeric/Prob/ConstantDistribution.h>
 #include <Bpp/Numeric/DataTable.h>
@@ -66,11 +68,16 @@ using namespace std;
 
 // From bpp-phyl:
 #include <Bpp/Phyl/Tree.h>
-#include <Bpp/Phyl/Likelihood.all>
+#include <Bpp/Phyl/Likelihood/RHomogeneousMixedTreeLikelihood.h>
+#include <Bpp/Phyl/Likelihood/DRHomogeneousMixedTreeLikelihood.h>
+#include <Bpp/Phyl/Likelihood/RNonHomogeneousMixedTreeLikelihood.h>
+#include <Bpp/Phyl/Likelihood/DRNonHomogeneousTreeLikelihood.h>
+#include <Bpp/Phyl/Likelihood/RASTools.h>
 #include <Bpp/Phyl/PatternTools.h>
 #include <Bpp/Phyl/App/PhylogeneticsApplicationTools.h>
 #include <Bpp/Phyl/OptimizationTools.h>
-#include <Bpp/Phyl/Model.all>
+#include <Bpp/Phyl/Model/SubstitutionModelSetTools.h>
+#include <Bpp/Phyl/Model/MixedSubstitutionModel.h>
 #include <Bpp/Phyl/Model/Protein/CoalaCore.h>
 #include <Bpp/Phyl/Model/RateDistribution/ConstantRateDistribution.h>
 #include <Bpp/Phyl/Model/FrequenciesSet/MvaFrequenciesSet.h>
@@ -93,9 +100,9 @@ void help()
 int main(int args, char** argv)
 {
   cout << "******************************************************************" << endl;
-  cout << "*       Bio++ Maximum Likelihood Computation, version 1.6.0      *" << endl;
+  cout << "*       Bio++ Maximum Likelihood Computation, version " << BPP_VERSION << "      *" << endl;
   cout << "*                                                                *" << endl;
-  cout << "* Authors: J. Dutheil                       Last Modif. 29/01/13 *" << endl;
+  cout << "* Authors: J. Dutheil                       Last Modif. " << BPP_REL_DATE << " *" << endl;
   cout << "*          B. Boussau                                            *" << endl;
   cout << "*          L. Guéguen                                            *" << endl;
   cout << "*          M. Groussin                                           *" << endl;
@@ -114,7 +121,7 @@ int main(int args, char** argv)
     bppml.startTimer();
 
     Alphabet* alphabet = SequenceApplicationTools::getAlphabet(bppml.getParams(), "", false);
-    auto_ptr<GeneticCode> gCode;
+    unique_ptr<GeneticCode> gCode;
     CodonAlphabet* codonAlphabet = dynamic_cast<CodonAlphabet*>(alphabet);
     if (codonAlphabet) {
       string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppml.getParams(), "Standard", "", true, true);
@@ -239,7 +246,7 @@ int main(int args, char** argv)
     bool optimizeTopo = ApplicationTools::getBooleanParameter("optimization.topology", bppml.getParams(), false, "", true, 1);
     unsigned int nbBS = ApplicationTools::getParameter<unsigned int>("bootstrap.number", bppml.getParams(), 0, "", true, 1);
 
-    SubstitutionModel*    model    = 0;
+    TransitionModel*    model    = 0;
     SubstitutionModelSet* modelSet = 0;
     DiscreteDistribution* rDist    = 0;
 
@@ -247,7 +254,7 @@ int main(int args, char** argv)
     {
       if (nhOpt != "no")
         throw Exception("Topology estimation with NH model not supported yet, sorry :(");
-      model = PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, gCode.get(), sites, bppml.getParams());
+      model = PhylogeneticsApplicationTools::getTransitionModel(alphabet, gCode.get(), sites, bppml.getParams());
       if (model->getName() != "RE08") SiteContainerTools::changeGapsToUnknownCharacters(*sites);
       if (model->getNumberOfStates() >= 2 * model->getAlphabet()->getSize())
       {
@@ -265,7 +272,7 @@ int main(int args, char** argv)
     }
     else if (nhOpt == "no")
     {
-      model = PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, gCode.get(), sites, bppml.getParams());
+      model = PhylogeneticsApplicationTools::getTransitionModel(alphabet, gCode.get(), sites, bppml.getParams());
       if (model->getName() != "RE08") SiteContainerTools::changeGapsToUnknownCharacters(*sites);
       if (model->getNumberOfStates() >= 2 * model->getAlphabet()->getSize())
       {
@@ -307,7 +314,7 @@ int main(int args, char** argv)
     }
     else if (nhOpt == "one_per_branch")
     {
-      model = PhylogeneticsApplicationTools::getSubstitutionModel(alphabet, gCode.get(), sites, bppml.getParams());
+      model = PhylogeneticsApplicationTools::getTransitionModel(alphabet, gCode.get(), sites, bppml.getParams());
       if (model->getName() != "RE08") SiteContainerTools::changeGapsToUnknownCharacters(*sites);
       if (model->getNumberOfStates() >= 2 * model->getAlphabet()->getSize())
       {
@@ -322,16 +329,18 @@ int main(int args, char** argv)
       if (model->getNumberOfStates() != alphabet->getSize())
       {
         // Markov-Modulated Markov Model...
-        unsigned int n = (unsigned int)(model->getNumberOfStates() / alphabet->getSize());
+        unsigned int n = static_cast<unsigned int>(model->getNumberOfStates() / alphabet->getSize());
         rateFreqs = vector<double>(n, 1. / static_cast<double>(n)); // Equal rates assumed for now, may be changed later (actually, in the most general case,
                                                        // we should assume a rate distribution for the root also!!!
       }
 
       bool stationarity = ApplicationTools::getBooleanParameter("nonhomogeneous.stationarity", bppml.getParams(), false, "", false, 1);
       FrequenciesSet* rootFreqs = 0;
+      std::map<std::string, std::string> aliasFreqNames;
       if (!stationarity)
       {
-        rootFreqs = PhylogeneticsApplicationTools::getRootFrequenciesSet(alphabet, gCode.get(), sites, bppml.getParams(), rateFreqs);
+        
+        rootFreqs = PhylogeneticsApplicationTools::getRootFrequenciesSet(alphabet, gCode.get(), sites, bppml.getParams(), aliasFreqNames, rateFreqs);
         stationarity = !rootFreqs;
         string freqDescription = ApplicationTools::getStringParameter("nonhomogeneous.root_freq", bppml.getParams(), "", "", true, 1);
         if (freqDescription == "MVAprotein")
@@ -348,9 +357,9 @@ int main(int args, char** argv)
       ApplicationTools::displayBooleanResult("Stationarity assumed", stationarity);
    
       vector<string> globalParameters = ApplicationTools::getVectorParameter<string>("nonhomogeneous_one_per_branch.shared_parameters", bppml.getParams(), ',', "");
-      for (unsigned int i = 0; i < globalParameters.size(); i++)
+      for (size_t i = 0; i < globalParameters.size(); i++)
         ApplicationTools::displayResult("Global parameter", globalParameters[i]);
-      modelSet = SubstitutionModelSetTools::createNonHomogeneousModelSet(model, rootFreqs, tree, globalParameters);
+      modelSet = SubstitutionModelSetTools::createNonHomogeneousModelSet(model, rootFreqs, tree, aliasFreqNames, globalParameters);
       model = 0;
 
       string recursion = ApplicationTools::getStringParameter("likelihood.recursion", bppml.getParams(), "simple", "", true, 1);
@@ -375,7 +384,9 @@ int main(int args, char** argv)
     else if (nhOpt == "general")
     {
       modelSet = PhylogeneticsApplicationTools::getSubstitutionModelSet(alphabet, gCode.get(), sites, bppml.getParams());
+
       if (modelSet->getModel(0)->getName() != "RE08") SiteContainerTools::changeGapsToUnknownCharacters(*sites);
+      
       if (modelSet->getNumberOfStates() >= 2 * modelSet->getAlphabet()->getSize())
       {
         // Markov-modulated Markov model!
@@ -415,7 +426,7 @@ int main(int args, char** argv)
       ApplicationTools::displayResult("List parameters to", paramNameFile);
       ofstream pnfile(paramNameFile.c_str(), ios::out);
       ParameterList pl = tl->getParameters();
-      for (unsigned int i = 0; i < pl.size(); ++i) {
+      for (size_t i = 0; i < pl.size(); ++i) {
         pnfile << pl[i].getName() << endl;
       }
       pnfile.close();
@@ -425,7 +436,7 @@ int main(int args, char** argv)
 
     //Check initial likelihood:
     double logL = tl->getValue();
-    if (isinf(logL))
+    if (std::isinf(logL))
     {
       // This may be due to null branch lengths, leading to null likelihood!
       ApplicationTools::displayWarning("!!! Warning!!! Initial likelihood is zero.");
@@ -440,7 +451,7 @@ int main(int args, char** argv)
       logL = tl->getValue();
     }
     ApplicationTools::displayResult("Initial log likelihood", TextTools::toString(-logL, 15));
-    if (isinf(logL))
+    if (std::isinf(logL))
     {
       ApplicationTools::displayError("!!! Unexpected initial likelihood == 0.");
       if (codonAlphabet)
@@ -448,7 +459,7 @@ int main(int args, char** argv)
         bool f = false;
         size_t s;
         for (size_t i = 0; i < sites->getNumberOfSites(); i++) {
-          if (isinf(tl->getLogLikelihoodForASite(i))) {
+          if (std::isinf(tl->getLogLikelihoodForASite(i))) {
             const Site& site = sites->getSite(i);
             s = site.size();
             for (size_t j = 0; j < s; j++) {
@@ -477,7 +488,7 @@ int main(int args, char** argv)
       } else {
         ApplicationTools::displayBooleanResult("Saturated site removal enabled", true);
         for (size_t i = sites->getNumberOfSites(); i > 0; --i) {
-          if (isinf(tl->getLogLikelihoodForASite(i - 1))) {
+          if (std::isinf(tl->getLogLikelihoodForASite(i - 1))) {
             ApplicationTools::displayResult("Ignore saturated site", sites->getSite(i - 1).getPosition());
             sites->deleteSite(i - 1);
           }
@@ -486,7 +497,7 @@ int main(int args, char** argv)
         tl->setData(*sites);
         tl->initialize();
         logL = tl->getValue();
-        if (isinf(logL)) {
+        if (std::isinf(logL)) {
           throw Exception("Likelihood is still 0 after saturated sites are removed! Looks like a bug...");
          }
         ApplicationTools::displayResult("Initial log likelihood", TextTools::toString(-logL, 15));
@@ -502,12 +513,12 @@ int main(int args, char** argv)
     // Write parameters to screen:
     ApplicationTools::displayResult("Log likelihood", TextTools::toString(-tl->getValue(), 15));
     ParameterList parameters = tl->getSubstitutionModelParameters();
-    for (unsigned int i = 0; i < parameters.size(); i++)
+    for (size_t i = 0; i < parameters.size(); i++)
     {
       ApplicationTools::displayResult(parameters[i].getName(), TextTools::toString(parameters[i].getValue()));
     }
     parameters = tl->getRateDistributionParameters();
-    for (unsigned int i = 0; i < parameters.size(); i++)
+    for (size_t i = 0; i < parameters.size(); i++)
     {
       ApplicationTools::displayResult(parameters[i].getName(), TextTools::toString(parameters[i].getValue()));
     }
@@ -517,6 +528,8 @@ int main(int args, char** argv)
 
     // Write parameters to file:
     string parametersFile = ApplicationTools::getAFilePath("output.estimates", bppml.getParams(), false, false, "none", 1);
+    bool withAlias = ApplicationTools::getBooleanParameter("output.estimates.alias", bppml.getParams(), true, "", true, 0);
+
     ApplicationTools::displayResult("Output estimates to file", parametersFile);
     if (parametersFile != "none")
     {
@@ -533,17 +546,17 @@ int main(int args, char** argv)
       if (modelSet)
       {
         modelSet->matchParametersValues(tl->getParameters());
-        PhylogeneticsApplicationTools::printParameters(modelSet, out);
+        PhylogeneticsApplicationTools::printParameters(modelSet, out, 1, withAlias);
       }
       else
       {
         model->matchParametersValues(tl->getParameters());
-        PhylogeneticsApplicationTools::printParameters(model, out);
+        PhylogeneticsApplicationTools::printParameters(model, out, 1, withAlias);
       }
       out.endLine();
       (out << "# Rate distribution parameters:").endLine();
       rDist->matchParametersValues(tl->getParameters());
-      PhylogeneticsApplicationTools::printParameters(rDist, out);
+      PhylogeneticsApplicationTools::printParameters(rDist, out, withAlias);
     }
 
     // Getting posterior rate class distribution:
