@@ -44,6 +44,8 @@ knowledge of the CeCILL license and that you accept its terms.
 
 using namespace std;
 
+// From bpp-core:
+#include <Bpp/Version.h>
 #include <Bpp/App/BppApplication.h>
 #include <Bpp/App/ApplicationTools.h>
 #include <Bpp/Io/FileTools.h>
@@ -55,11 +57,10 @@ using namespace std;
 #include <Bpp/Seq/Alphabet/Alphabet.h>
 #include <Bpp/Seq/Alphabet/AlphabetTools.h>
 #include <Bpp/Seq/Container/VectorSiteContainer.h>
+#include <Bpp/Seq/Container/SiteContainerTools.h>
+#include <Bpp/Seq/Container/SequenceContainerTools.h>
 #include <Bpp/Seq/App/SequenceApplicationTools.h>
-#include <Bpp/Seq/Io.all>
-#include <Bpp/Seq/Container.all>
 #include <Bpp/Seq/SequenceTools.h>
-#include <Bpp/Seq/GeneticCode.all>
 
 //From bpp-phyl:
 #include <Bpp/Phyl/Tree.h>
@@ -80,8 +81,8 @@ void help()
 int main(int args, char** argv)
 {
   cout << "******************************************************************" << endl;
-  cout << "*           Bio++ Sequence Manipulator, version 2.2.0.           *" << endl;
-  cout << "* Author: J. Dutheil                        Last Modif. 25/09/14 *" << endl;
+  cout << "*           Bio++ Sequence Manipulator, version " << BPP_VERSION << ".           *" << endl;
+  cout << "* Author: J. Dutheil                        Last Modif. " << BPP_REL_DATE << " *" << endl;
   cout << "******************************************************************" << endl;
   cout << endl;
   
@@ -98,22 +99,31 @@ int main(int args, char** argv)
   
   // Get alphabet
   Alphabet* alphabet = SequenceApplicationTools::getAlphabet(bppseqman.getParams(), "", false, true, true);
-  auto_ptr<GeneticCode> gCode;
+  unique_ptr<GeneticCode> gCode;
   CodonAlphabet* codonAlphabet = dynamic_cast<CodonAlphabet*>(alphabet);
 
   // Get sequences:
-  SequenceContainer* tmp = SequenceApplicationTools::getSequenceContainer(alphabet, bppseqman.getParams(), "", true, true);
-  OrderedSequenceContainer* sequences = new VectorSequenceContainer(*tmp);
-  delete tmp;
+  bool aligned = ApplicationTools::getBooleanParameter("input.alignment", bppseqman.getParams(), false, "", true, 1);
+  OrderedSequenceContainer* sequences = 0;
+
+  if (aligned) {
+    VectorSiteContainer* allSites = SequenceApplicationTools::getSiteContainer(alphabet, bppseqman.getParams());
+    sequences = SequenceApplicationTools::getSitesToAnalyse(*allSites, bppseqman.getParams(), "", true, false);
+    delete allSites;
+  } else {
+    SequenceContainer* tmp = SequenceApplicationTools::getSequenceContainer(alphabet, bppseqman.getParams(), "", true, true);
+    sequences = new VectorSequenceContainer(*tmp);
+    delete tmp;
+  }
+
   ApplicationTools::displayResult("Number of sequences", sequences->getNumberOfSequences());
   
   // Perform manipulations
   
   vector<string> actions = ApplicationTools::getVectorParameter<string>("sequence.manip", bppseqman.getParams(), ',', "", "", false, 1);
   
-  bool aligned = false;
 
-  for (unsigned int a = 0; a < actions.size(); a++)
+  for (size_t a = 0; a < actions.size(); a++)
   {
     string cmdName;
     map<string, string> cmdArgs;
@@ -128,7 +138,7 @@ int main(int args, char** argv)
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
       else         sc = new VectorSequenceContainer(sequences->getAlphabet());
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         Sequence* seq = SequenceTools::getComplement(sequences->getSequence(i));
         sc->addSequence(*seq, false);
@@ -190,10 +200,13 @@ int main(int args, char** argv)
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(alpha);
       else         sc = new VectorSequenceContainer(alpha);
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         const Sequence* old = &sequences->getSequence(i);
-        Sequence* seq = new BasicSequence(old->getName(), old->getContent(), old->getComments(), alpha);
+        vector<int> content(old->size());
+        for (size_t j = 0; j < old->size(); ++j)
+          content[j] = (*old)[j];
+        Sequence* seq = new BasicSequence(old->getName(), content, old->getComments(), alpha);
         sc->addSequence(*seq, false);
         delete seq;
       }
@@ -233,9 +246,9 @@ int main(int args, char** argv)
     else if (cmdName == "RemoveGaps")
     {
       VectorSequenceContainer* sc = new VectorSequenceContainer(sequences->getAlphabet());
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); i++)
       {
-        auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
+        unique_ptr<Sequence> seq(sequences->getSequence(i).clone());
         SequenceTools::removeGaps(*seq);
         sc->addSequence(*seq);
       }
@@ -251,7 +264,7 @@ int main(int args, char** argv)
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
       else         sc = new VectorSequenceContainer(sequences->getAlphabet());
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         Sequence* seq = new BasicSequence(sequences->getSequence(i));
         SymbolListTools::changeGapsToUnknownCharacters(*seq);
@@ -269,7 +282,7 @@ int main(int args, char** argv)
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
       else         sc = new VectorSequenceContainer(sequences->getAlphabet());
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         Sequence* seq = new BasicSequence(sequences->getSequence(i));
         SymbolListTools::changeUnresolvedCharactersToGaps(*seq);
@@ -296,7 +309,7 @@ int main(int args, char** argv)
         VectorSequenceContainer* sc = new VectorSequenceContainer(sequences->getAlphabet());
         for (size_t i = 0; i < sequences->getNumberOfSequences(); ++i)
         {
-          auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
+          unique_ptr<Sequence> seq(sequences->getSequence(i).clone());
           SequenceTools::removeStops(*seq, *gCode);
           sc->addSequence(*seq);
         }
@@ -306,7 +319,7 @@ int main(int args, char** argv)
         VectorSiteContainer* sc = new VectorSiteContainer(sequences->getAlphabet());
         for (size_t i = 0; i < sequences->getNumberOfSequences(); ++i)
         {
-          auto_ptr<Sequence> seq(sequences->getSequence(i).clone());
+          unique_ptr<Sequence> seq(sequences->getSequence(i).clone());
           SequenceTools::replaceStopsWithGaps(*seq, *gCode);
           sc->addSequence(*seq);
         }
@@ -318,12 +331,12 @@ int main(int args, char** argv)
     // +--------------+
     // | Remove stops |
     // +--------------+
-    else if (cmdName == "RemoveColumnsWithStop")
+    else if (cmdName == "RemoveColumnsWithStops")
     {
       SiteContainer* sites = dynamic_cast<SiteContainer*>(sequences);
       if (!sites)
       {
-        throw Exception("'RemoveColumnsWithStop' can only be used on alignment. You may consider using the 'CoerceToAlignment' command.");
+        throw Exception("'RemoveColumnsWithStops' can only be used on alignment. You may consider using the 'CoerceToAlignment' command.");
       }
       if (!gCode.get()) {
         string codeDesc = ApplicationTools::getStringParameter("genetic_code", bppseqman.getParams(), "Standard", "", true, 1);
@@ -440,7 +453,7 @@ int main(int args, char** argv)
       OrderedSequenceContainer* sc = 0;
       if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
       else         sc = new VectorSequenceContainer(sequences->getAlphabet());
-      for (unsigned int i = 0; i < sequences->getNumberOfSequences(); i++)
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); i++)
       {
         const Sequence* old = &sequences->getSequence(i);
         Sequence* seq = SequenceTools::getInvert(*old);
@@ -470,7 +483,7 @@ int main(int args, char** argv)
     // +-----------------+
     else if (cmdName == "FilterFromTree")
     {
-      auto_ptr<Tree> tree(PhylogeneticsApplicationTools::getTree(cmdArgs, ""));
+      unique_ptr<Tree> tree(PhylogeneticsApplicationTools::getTree(cmdArgs, ""));
       vector<string> names = tree->getLeavesNames();
       OrderedSequenceContainer* reorderedSequences = 0;
       if (aligned) {
@@ -484,6 +497,22 @@ int main(int args, char** argv)
       delete sequences;
       sequences = reorderedSequences;
     }
+    // +----------------------+
+    // | RemoveEmptySequences |
+    // +----------------------+
+    else if (cmdName == "RemoveEmptySequences")
+    {
+      OrderedSequenceContainer* sc = 0;
+      if (aligned) sc = new VectorSiteContainer(sequences->getAlphabet());
+      else         sc = new VectorSequenceContainer(sequences->getAlphabet());
+      for (size_t i = 0; i < sequences->getNumberOfSequences(); ++i)
+      {
+        if (SequenceTools::getNumberOfSites(sequences->getSequence(i))!=0)
+          sc->addSequence(sequences->getSequence(i), false);
+      }
+      delete sequences;
+      sequences = sc;
+    }
 
     else throw Exception("Unknown action: " + cmdName);
   }
